/* view2.c 

   Laura Toma 
   
   
   has a function to draw a 3D cube, called after translate/rotate/scale transformations. 
   
   use it to demonstrate the order of transformations


   viewed in 3D using the default projection.

*/

#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <assert.h>

#ifdef __APPLE__
#include <GLUT/glut.h>
#else
#include <GL/glut.h>
#endif


/* forward declarations of functions */
void display(void);
void renderScene(void);
void keypress(unsigned char key, int x, int y);
void main_menu(int value);



//draw a rectangle [-1,1]x[-1,1]  at depth z, with given colors
void draw_xy_rect(GLfloat z, GLfloat r, GLfloat g, GLfloat b) {

  glBegin(GL_POLYGON);
  glColor3f(r,g,b);
  glVertex3f(-1,-1, z);
  glVertex3f(-1,1, z);
  glVertex3f(1,1, z);
  glVertex3f(1,-1, z);
  glEnd();
}

//draw cube [-1, 1] x [-1,1]  x [z1, z2]
void draw_cube(GLfloat z1, GLfloat z2) {
  glPolygonMode(GL_FRONT_AND_BACK, GL_LINE); 
  glColor3f(1.0, 1.0, 1.0);

  //face at depth z1
  draw_xy_rect(z1, 1,0,0);

  //face at depth z2
  draw_xy_rect(z2, 0,0,1);
  
  //side face on the left 
  glBegin(GL_POLYGON);
  glVertex3f(-1,-1, z1);
  glVertex3f(-1,1, z1);
  glVertex3f(-1,1, z2);
  glVertex3f(-1,-1, z2);
  glEnd();

  //side face on the right 
  glBegin(GL_POLYGON);
  glVertex3f(1,-1, z1);
  glVertex3f(1,1, z1);
  glVertex3f(1,1, z2);
  glVertex3f(1,-1, z2);
  glEnd();

}


void keypress(unsigned char key, int x, int y) {
  switch(key) {
  case 'q':
    exit(0);
    break;
  } 
}



void display(void) {
  draw_cube(-0.5, -0.7);
}


void renderScene(void) {
  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);


  /* local coordinate system: rotate object and its coordinate
     system 45 deg around z-axis, then translate  */
  glRotatef(45.0, 0.0,0.0,1.0);
  glTranslatef(0.5, 0, 0);  
 
  //change the order of transformations and see what happens. we need
  //to understand transformations so that we know precisely how to place
  //an object in a scece.


  glScalef(0.5,0.5,0.5);
  display();
  
  /* execute the drawing commands */
  glFlush();
}




int main(int argc, char** argv) {

    /* open a window */
  glutInit(&argc, argv);
  glutInitDisplayMode(GLUT_SINGLE | GLUT_RGB | GLUT_DEPTH);
  glutInitWindowSize(500, 500);
  glutInitWindowPosition(100,100);
  glutCreateWindow(argv[0]);

  /* OpenGL init */
  glClearColor(0, 0, 0, 0);   /* set background color black*/

  /* callback functions */
  glutDisplayFunc(renderScene); 
  glutKeyboardFunc(keypress);

  /* event handler */
  glutMainLoop();
  return 0;
}

